#include "Arduino.h"
#define DEBUG //uncomment for debug printouts
#ifdef DEBUG
  #define dprint(x)      Serial.print(x) 
  #define dprintln(x)    Serial.println(x)
  #define dprintBIN(x)   Serial.print(x,BIN)
  #define dprintlnBIN(x) Serial.println(x,BIN)
  #define dprintHEX(x)   Serial.print(x,HEX)
  #define dprintlnHEX(x) Serial.println(x,HEX)
  #define dprintBegin(x) Serial.begin(x)
#else
#ifdef DEBUGBLE
  #define dprint(x)      bleKeyb.print(x)
  #define dprintln(x)    bleKeyb.println(x)
  #define dprintBIN(x)   bleKeyb.print(x)
  #define dprintlnBIN(x) bleKeyb.println(x)
  #define dprintHEX(x)   bleKeyb.print(x)
  #define dprintlnHEX(x) bleKeyb.println(x)
  #define dprintBegin(x)
#else
  #define dprint(x)
  #define dprintln(x)
  #define dprintBIN(x) 
  #define dprintlnBIN(x)
  #define dprintHEX(x)  
  #define dprintlnHEX(x) 
  #define dprintBegin(x)
#endif
#endif

#include "Declarations.h"
#include "tunes.h"
#include "fSupport.h"
#include "fCore.h"

void setup() {
  dprintBegin(115200);  dprintln("------start");
  //pinMode(13, OUTPUT);     //--Built in RED LED

  //---Buzzer---
  pinMode(buzzPin, OUTPUT); 
  //ledcSetup(buzzChan, 10000, 12); //--Parms=(channel, freq, resolution)
  ledcAttachPin(buzzPin, buzzChan);

  //---LEDs---
  pinMode(latchPin, OUTPUT);
  pinMode(dataPin, OUTPUT);
  pinMode(clockPin, OUTPUT);
  send2Shift(B11000000,B11111111);  //all LEDs off

  //---Preferences---
  storedVars.begin("baseVars", false);
  //bleMode = storedVars.getBool("bleModeTemp",true);
  symbolTable = storedVars.getUChar("symbolTableTemp",1);
  //dprint("FLASH bleMode = "); dprintln(bleMode);
  dprint("FLASH symTable = "); dprintln(symbolTable);
  storedVars.end();

  //---NeoPixel---
  pinMode(NEOPIXEL_I2C_POWER, OUTPUT); // GPIO 2
  digitalWrite(NEOPIXEL_I2C_POWER, HIGH);
  pinMode(0, OUTPUT); 
  pix.begin();
  pix.setBrightness(5);
  yellow = pix.Color(255,255,0);  orange = pix.Color(255,50,0);
  blue = pix.Color(0,150,255);    red = pix.Color(255,0,0);
  green = pix.Color(0,255,0);     violet = pix.Color(255,0,255);

  //---Cap Sensors---
  resetTouchVars();
  //loadSymbols(symbolTable); 
  Wire.begin (22,20);
  Wire.setClock(400000); //MAX=400khz per MPR121 specs
  //Wire.setTimeOut(50); //default=50 ms
  dprint("I2C Clock: "); dprintln(Wire.getClock());
  dprint("Time Out : "); dprintln(Wire.getTimeOut());
  if (!S0.begin(0x5A)) {dprintln("0x5A Not Ok"); buzz(30); while (1); } else buzz(10);
  if (!S1.begin(0x5C)) {dprintln("0x5C Not Ok"); buzz(30); while (1); } else buzz(11);
  if (!S2.begin(0x5D)) {dprintln("0x5D Not Ok"); buzz(30); while (1); } else buzz(12);

  //---Functions Switches & Interrupts
  for (i=0; i<5; i++) {
    pinMode(FS[i].PIN,INPUT); 
    FS[i].value = digitalRead(FS[i].PIN); //---initial state of each switch
    if (FS[i].value) progMode = false;
    attachInterruptArg(FS[i].PIN,readInterrupts,&FS[i],CHANGE);
  }
  letterKeyb = FS[0].value; dprintln("\n0:letter: " + String(letterKeyb));
  wordKeyb = FS[1].value; dprintln("1:word: " + String(wordKeyb));
  phraseKeyb = FS[2].value; dprintln("2:phrase: " + String(phraseKeyb));
  ledKeyb = FS[3].value; dprintln("3:LED: " + String(ledKeyb));
  buzzKeyb = FS[4].value; dprintln("4:Buzz: " + String(buzzKeyb));

  //---BLE Keyboard---
  if (letterKeyb + wordKeyb + phraseKeyb == 0) {
    bleMode = false; dprintln("bleMode OFF in Setup");}
  else {  
    bleKeyb.setName("RPM Board GEN3");  
    bleKeyb.begin();
  }
  loadSymbols(symbolTable); //moved from CapSensors group to playTune at end of Setup
}

void loop() {
  if (!progMode && bleMode) {  // i.e. in runMode
    while (!bleKeyb.isConnected() && !progMode && bleMode) { 
      //progMode & bleMode could change via interrupts
      waitingOnBLE = true; 
      keybON = false; 
      buzz(20); 
      if (sFlag) chkSwitches();  //in case user enters progMode or 
                                 //turns BLE OFF while waiting for BLE pair    
    }
    waitingOnBLE = false;
    if (!keybON && bleKeyb.isConnected()) {  
      //run once after connected or reconnected
      keybON = true;
      bleKeyb.setDelay(3); //to compensate for fast input in iOS. Default=8
      buzz(21);
    }
  }    
  if (sFlag) chkSwitches();
  chkTouched();
  if (LEDisON) chkOnLED(); 
  chkBattery();
}



